/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package download;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.text.SimpleDateFormat;
import java.util.Properties;

/**
 *
 * @date 8-Nov-2011
 * @author Corneliu Bal & Cristian Tiron
 * incapsuleaza logica fiserului de configurare (parametri/proprietati)
 *  unic pe proces
 */
public class Params
{
    private static String _newLine = System.getProperty("line.separator");
    private static Properties _props = null;
    private static String _propFile = null;
    private static boolean _propsChanged = false;
    private static SimpleDateFormat _dateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
    private static String _partition = null;
    private static int _partitionModulo = -1;

    //clasa nu poate fi instantiata
    private Params()
    {
    }

    //transmitenumele fisierului de parametri
    public static void init(String paramsFile)
    {
        if(_props != null)
        {
            close();
        }
        _propFile = paramsFile;
        try
        {
            _props = new Properties();
            FileInputStream file = new FileInputStream(_propFile);
            _props.load(file);
            file.close();
        }
        catch(Throwable ex)
        {
            _props = null;
        }
        _dateFormat.setLenient(false);
    }

    //inchidere stream-uri log si trace
    public static void close()
    {
        if(_propsChanged == true)
        {
            FileOutputStream file = null;
            try
            {
                file = new FileOutputStream(_propFile);
                _props.store(file, "pentru a comenta o linie, cu pastrarea ei dupa salvare, inserati un 'x' la inceputul liniei");
            }
            //catch exception in case properties file does not exist
            catch(Throwable e)
            {
            }
            finally
            {
                try
                {
                    file.close();
                }
                catch(Throwable e)
                {
                }
            }
        }
        _partition = null;
        _partitionModulo = -1;
        _propFile = null;
        _props = null;
        _propsChanged = false;
    }

    //obtine o proprietate cu specificarea unei valori prin lipsa
    public static String getProperty(String name, String defValue)
    {
        if(_props == null)
        {
            return null;
        }
        try
        {
            return _props.getProperty(name, defValue);
        }
        catch(Throwable ex)
        {
        }
        return null;
    }

    //obtine o proprietate
    public static String getProperty(String name)
    {
        if(_props == null)
        {
            return null;
        }
        try
        {
            return _props.getProperty(name);
        }
        catch(Throwable ex)
        {
        }
        return null;
    }

    //modifica/creaza o proprietate
    public static String setProperty(String name, String value)
    {
        String val = null;
        if(_props == null)
        {
            return null;
        }
        try
        {
            Object obj = _props.setProperty(name, value);
            if(obj != null)
            {
                val = obj.toString();
            }
            _propsChanged = true;
            return val;
        }
        catch(Throwable ex)
        {
        }
        return null;
    }

    //stabileste calea in FileSystem-ul de pe server
    //path este o cale citita din fisierul de configurare, care poate
    //  contine urmatoarele place-holder-e:
    //  - %p - numele partitiei (FileSysytem) in care se stocheaza fisierul
    //  - %s - nume subfolder (ultimele 2 pozitii din id)
    //Repartitia fisierului asociat cu parametrul id intr-o partitie se face
    //  pe baza parametrului"partitionare" din fisierul de configurare:
    //  partitionare=nume:m
    //unde:
    //  - m este numarul de partitii
    //  - nume este numele partitiei, care va trebui sa contina place-holder-ul
    //      %i, care se va inlocui cu numarul (0 - m-1) reprezentand restul
    //      impartitii lui id la m
    //daca parametrul "partitionare" lipseste place-holder-ul %p va fi ignorat
    //parametrul ext poate fi:
    //  - null - se intoarce path dupa substituire
    //  - "" - se adauga la path anterior stringul <id>
    //  - altceva - se adauga la path anterior stringul <ext>
    public static String establishPath(String path, long id, String ext)
    {
        String part = null;
        //obtinerea initiala a parametrului "partitionare"
        if(_partitionModulo < 0)
        {
            part = getProperty("partitionare");
            if(part == null)
            {
                _partitionModulo = 0;
            }
            else
            {
                try
                {
                    String[] parts = part.split(":");
                    if(parts.length != 2)
                    {
                        _partitionModulo = 0;
                    }
                    else
                    {
                        _partitionModulo = Integer.parseInt(parts[1]);
                        _partition = parts[0];
                        if(_partition.indexOf("%i") < 0)
                        {
                            _partitionModulo = 0;
                        }
                    }
                }
                catch(Throwable ex)
                {
                    _partitionModulo = 0;
                }
            }
            if(_partitionModulo == 0)
            {
                _partition = "";
            }
        }
        try
        {
            int n = 0;
            //substituire place_holder %p
            if(_partitionModulo > 0)
            {
                n = (int)(id % _partitionModulo);
                part = ((n < 10) ? "0" : "") + Integer.toString(n);
                path = path.replaceAll("%p", _partition.replaceAll("%i", part));
            }
            //substituire place_holder %s
            n = (int)(id % 100);
            part = ((n < 10) ? "0" : "") + Integer.toString(n);
            path = path.replaceAll("%s", part);
            if(ext == null)
            {
                return path;
            }
            path = padPath(path);
            path += Long.toString(id);
            if(ext.equals("") == true)
            {
                return path;
            }
            return path + ext;
        }
        catch(Throwable ex)
        {
        }
        return null;
    }

    public static String establishPath(String path, long id)
    {
        return establishPath(path, id, null);
    }

    private static String padPath(String path)
    {
        if(path.endsWith("/") == false && path.endsWith("\\") == false)
        {
            return path + File.separator;
        }
        return path;
    }
}
